/*
Supports filtering call data for safe logging, for example to strip passwords
or credit card numbers.

Example usage: FAAS_IX_LOGGING_FILTER=./api/logging_filter.js ruby node_modules/faas/bin/faas-ix-server-ruby api/v0/

Contents of api/logging_filter.js:
----------------------------------

exports.filter = function(callData){
  if (callData.api === 'Api.V0.Auth.login') {
    callData.args[1] = '[REDACTED]'
  }
  return callData
}

exports.getKeyFilterRegexes = function(oldRegexes){
  oldRegexes.push(/name/)
  return oldRegexes
}

*/


var Util = require('util')
  , Path = require('path')
  , Base = require('../base')

var debug = Base.logger('ix-logging_filter')


var LOGGING_FILTER = null
var FILTERED_KEYS = [ /password/i ]


exports.loadFilter = function(callback){
  Base.env(function(){
    var filterPath = Base.env.IX_LOGGING_FILTER;
    if (filterPath) {
      console.log('Loading logging filter '+filterPath)
      try {
        LOGGING_FILTER = require(filterPath)
      }
      catch (e) {
        try {
          LOGGING_FILTER = require(Path.join(process.cwd(), filterPath))
        }
        catch (e2) {
          console.log("Unable to load logging filter from "+filterPath+": "+e2+ ' (working dir: '+process.cwd()+')')
        }
      }
    }
    if (LOGGING_FILTER && LOGGING_FILTER.getKeyFilterRegexes) {
      try {
        FILTERED_KEYS = LOGGING_FILTER.getKeyFilterRegexes(FILTERED_KEYS)
      }
      catch (e) {
        console.log('Error setting logging filters: '+e)
      }
    }
    callback()
  })
}

exports.logCall = function(callData){
  try {
    var filteredCall = filterCall(callData)
    console.log(JSON.stringify(filteredCall))
  }
  catch(e){}
}

var filterCall = function(callData){
  var filtered
  try {
    // Deep-dup so that we don't modify the actual args.
    filtered = JSON.parse(JSON.stringify(callData))
    filteredArgs = filtered.args || []
    filteredArgs = filterDataHelper(filteredArgs)
    filtered.args = filteredArgs
    if (LOGGING_FILTER && LOGGING_FILTER.filter) {
      filtered = LOGGING_FILTER.filter(filtered)
    }
  }
  catch(e){
    console.log("Error in logging filter: "+e)
  }
  return filtered
}

var filterDataHelper = function(arg, prefix){
  var i, key
  var newPrefix, matched
  if (Util.isArray(arg)) {
    for (i = 0; i < arg.length; i++) {
      arg[i] = filterDataHelper(arg[i])
    }
  }
  else if (typeof arg === 'object') {
    for (key in arg) { if (arg.hasOwnProperty(key)) {
      newPrefix = prefix || ''
      if (newPrefix) { newPrefix += '.' }
      newPrefix = newPrefix + key
      matched = matchedFilter(newPrefix)
      if (matched) {
        arg[key] = '[FILTERED:'+matched+']'
      }
      else {
        arg[key] = filterDataHelper(arg[key], newPrefix)
      }
    } }
  }
  // else arg doesn't change.
  return arg
}

var matchedFilter = function(key){
  var i, re
  for (i = 0; i < FILTERED_KEYS.length; i++) {
    re = FILTERED_KEYS[i]
    if (key.match(re)) {
      return re
    }
  }
  return null
}
